/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

/** @module vertx-web-js/jwt_auth_handler */
var utils = require('vertx-js/util/utils');
var AuthHandler = require('vertx-web-js/auth_handler');
var User = require('vertx-auth-common-js/user');
var RoutingContext = require('vertx-web-js/routing_context');
var JWTAuth = require('vertx-auth-jwt-js/jwt_auth');

var io = Packages.io;
var JsonObject = io.vertx.core.json.JsonObject;
var JJWTAuthHandler = Java.type('io.vertx.ext.web.handler.JWTAuthHandler');

/**
 An auth handler that provides JWT Authentication support.

 @class
*/
var JWTAuthHandler = function(j_val) {

  var j_jWTAuthHandler = j_val;
  var that = this;
  AuthHandler.call(this, j_val);

  var __super_handle = this.handle;
  var __super_addAuthority = this.addAuthority;
  var __super_addAuthorities = this.addAuthorities;
  var __super_parseCredentials = this.parseCredentials;
  var __super_authorize = this.authorize;
  var __super_create = this.create;
  var __super_create = this.create;
  var __super_setAudience = this.setAudience;
  var __super_setIssuer = this.setIssuer;
  var __super_setIgnoreExpiration = this.setIgnoreExpiration;
  /**
   Something has happened, so handle it.

   @public
   @param event {RoutingContext} the event to handle 
   */
  this.handle =  function(event) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'object' && __args[0]._jdel) {
      j_jWTAuthHandler["handle(io.vertx.ext.web.RoutingContext)"](__args[0]._jdel);
    } else if (typeof __super_handle != 'undefined') {
      return __super_handle.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Add a required authority for this auth handler

   @public
   @param authority {string} the authority 
   @return {AuthHandler} a reference to this, so the API can be used fluently
   */
  this.addAuthority =  function(authority) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'string') {
      j_jWTAuthHandler["addAuthority(java.lang.String)"](__args[0]) ;
      return that;
    } else if (typeof __super_addAuthority != 'undefined') {
      return __super_addAuthority.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Add a set of required authorities for this auth handler

   @public
   @param authorities {Array.<string>} the set of authorities 
   @return {AuthHandler} a reference to this, so the API can be used fluently
   */
  this.addAuthorities =  function(authorities) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'object' && __args[0] instanceof Array) {
      j_jWTAuthHandler["addAuthorities(java.util.Set)"](utils.convParamSetBasicOther(__args[0])) ;
      return that;
    } else if (typeof __super_addAuthorities != 'undefined') {
      return __super_addAuthorities.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Parses the credentials from the request into a JsonObject. The implementation should
   be able to extract the required info for the auth provider in the format the provider
   expects.

   @public
   @param context {RoutingContext} the routing context 
   @param handler {function} the handler to be called once the information is available. 
   */
  this.parseCredentials =  function(context, handler) {
    var __args = arguments;
    if (__args.length === 2 && typeof __args[0] === 'object' && __args[0]._jdel && typeof __args[1] === 'function') {
      j_jWTAuthHandler["parseCredentials(io.vertx.ext.web.RoutingContext,io.vertx.core.Handler)"](__args[0]._jdel, function(ar) {
        if (ar.succeeded()) {
          __args[1](utils.convReturnJson(ar.result()), null);
        } else {
          __args[1](null, ar.cause());
        }
      });
    } else if (typeof __super_parseCredentials != 'undefined') {
      return __super_parseCredentials.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Authorizes the given user against all added authorities.

   @public
   @param user {User} a user. 
   @param handler {function} the handler for the result. 
   */
  this.authorize =  function(user, handler) {
    var __args = arguments;
    if (__args.length === 2 && typeof __args[0] === 'object' && __args[0]._jdel && typeof __args[1] === 'function') {
      j_jWTAuthHandler["authorize(io.vertx.ext.auth.User,io.vertx.core.Handler)"](__args[0]._jdel, function(ar) {
        if (ar.succeeded()) {
          __args[1](null, null);
        } else {
          __args[1](null, ar.cause());
        }
      });
    } else if (typeof __super_authorize != 'undefined') {
      return __super_authorize.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Set the audience list

   @public
   @param audience {Array.<string>} the audience list 
   @return {JWTAuthHandler} a reference to this for fluency
   */
  this.setAudience =  function(audience) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'object' && __args[0] instanceof Array) {
      j_jWTAuthHandler["setAudience(java.util.List)"](utils.convParamListBasicOther(__args[0])) ;
      return that;
    } else if (typeof __super_setAudience != 'undefined') {
      return __super_setAudience.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Set the issuer

   @public
   @param issuer {string} the issuer 
   @return {JWTAuthHandler} a reference to this for fluency
   */
  this.setIssuer =  function(issuer) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] === 'string') {
      j_jWTAuthHandler["setIssuer(java.lang.String)"](__args[0]) ;
      return that;
    } else if (typeof __super_setIssuer != 'undefined') {
      return __super_setIssuer.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  /**
   Set whether expiration is ignored

   @public
   @param ignoreExpiration {boolean} whether expiration is ignored 
   @return {JWTAuthHandler} a reference to this for fluency
   */
  this.setIgnoreExpiration =  function(ignoreExpiration) {
    var __args = arguments;
    if (__args.length === 1 && typeof __args[0] ==='boolean') {
      j_jWTAuthHandler["setIgnoreExpiration(boolean)"](__args[0]) ;
      return that;
    } else if (typeof __super_setIgnoreExpiration != 'undefined') {
      return __super_setIgnoreExpiration.apply(this, __args);
    }
    else throw new TypeError('function invoked with invalid arguments');
  };

  // A reference to the underlying Java delegate
  // NOTE! This is an internal API and must not be used in user code.
  // If you rely on this property your code is likely to break if we change it / remove it without warning.
  this._jdel = j_jWTAuthHandler;
};

JWTAuthHandler._jclass = utils.getJavaClass("io.vertx.ext.web.handler.JWTAuthHandler");
JWTAuthHandler._jtype = {accept: function(obj) {
    return JWTAuthHandler._jclass.isInstance(obj._jdel);
  },wrap: function(jdel) {
    var obj = Object.create(JWTAuthHandler.prototype, {});
    JWTAuthHandler.apply(obj, arguments);
    return obj;
  },
  unwrap: function(obj) {
    return obj._jdel;
  }
};
JWTAuthHandler._create = function(jdel) {var obj = Object.create(JWTAuthHandler.prototype, {});
  JWTAuthHandler.apply(obj, arguments);
  return obj;
}
/**
 Create a JWT auth handler

 @memberof module:vertx-web-js/jwt_auth_handler
 @param authProvider {JWTAuth} the auth provider to use. 
 @param skip {string} 
 @return {JWTAuthHandler} the auth handler
 */
JWTAuthHandler.create =  function() {
  var __args = arguments;
  if (__args.length === 1 && typeof __args[0] === 'object' && __args[0]._jdel) {
    return utils.convReturnVertxGen(JWTAuthHandler, JJWTAuthHandler["create(io.vertx.ext.auth.jwt.JWTAuth)"](__args[0]._jdel)) ;
  } else if (__args.length === 2 && typeof __args[0] === 'object' && __args[0]._jdel && typeof __args[1] === 'string') {
    return utils.convReturnVertxGen(JWTAuthHandler, JJWTAuthHandler["create(io.vertx.ext.auth.jwt.JWTAuth,java.lang.String)"](__args[0]._jdel, __args[1])) ;
  }else throw new TypeError('function invoked with invalid arguments');
};

module.exports = JWTAuthHandler;