/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

/** @module vertx-web-js/local_session_store */
var utils = require('vertx-js/util/utils');
var Session = require('vertx-web-js/session');
var SessionStore = require('vertx-web-js/session_store');
var Vertx = require('vertx-js/vertx');

var io = Packages.io;
var JsonObject = io.vertx.core.json.JsonObject;
var JLocalSessionStore = Java.type('io.vertx.ext.web.sstore.LocalSessionStore');

/**
 A session store which is only available on a single node.
 <p>
 Can be used when sticky sessions are being used.

 @class
*/
var LocalSessionStore = function(j_val) {

  var j_localSessionStore = j_val;
  var that = this;
  SessionStore.call(this, j_val);

  var __super_create = this.create;
  var __super_create = this.create;
  var __super_create = this.create;
  // A reference to the underlying Java delegate
  // NOTE! This is an internal API and must not be used in user code.
  // If you rely on this property your code is likely to break if we change it / remove it without warning.
  this._jdel = j_localSessionStore;
};

LocalSessionStore._jclass = utils.getJavaClass("io.vertx.ext.web.sstore.LocalSessionStore");
LocalSessionStore._jtype = {accept: function(obj) {
    return LocalSessionStore._jclass.isInstance(obj._jdel);
  },wrap: function(jdel) {
    var obj = Object.create(LocalSessionStore.prototype, {});
    LocalSessionStore.apply(obj, arguments);
    return obj;
  },
  unwrap: function(obj) {
    return obj._jdel;
  }
};
LocalSessionStore._create = function(jdel) {var obj = Object.create(LocalSessionStore.prototype, {});
  LocalSessionStore.apply(obj, arguments);
  return obj;
}
/**
 Create a session store

 @memberof module:vertx-web-js/local_session_store
 @param vertx {Vertx} the Vert.x instance 
 @param sessionMapName {string} name for map used to store sessions 
 @param reaperInterval {number} how often, in ms, to check for expired sessions 
 @return {LocalSessionStore} the session store
 */
LocalSessionStore.create =  function() {
  var __args = arguments;
  if (__args.length === 1 && typeof __args[0] === 'object' && __args[0]._jdel) {
    return utils.convReturnVertxGen(LocalSessionStore, JLocalSessionStore["create(io.vertx.core.Vertx)"](__args[0]._jdel)) ;
  } else if (__args.length === 2 && typeof __args[0] === 'object' && __args[0]._jdel && typeof __args[1] === 'string') {
    return utils.convReturnVertxGen(LocalSessionStore, JLocalSessionStore["create(io.vertx.core.Vertx,java.lang.String)"](__args[0]._jdel, __args[1])) ;
  } else if (__args.length === 3 && typeof __args[0] === 'object' && __args[0]._jdel && typeof __args[1] === 'string' && typeof __args[2] ==='number') {
    return utils.convReturnVertxGen(LocalSessionStore, JLocalSessionStore["create(io.vertx.core.Vertx,java.lang.String,long)"](__args[0]._jdel, __args[1], __args[2])) ;
  }else throw new TypeError('function invoked with invalid arguments');
};

LocalSessionStore.DEFAULT_SESSIONID_LENGTH = JLocalSessionStore.DEFAULT_SESSIONID_LENGTH;
LocalSessionStore.DEFAULT_REAPER_INTERVAL = JLocalSessionStore.DEFAULT_REAPER_INTERVAL;
LocalSessionStore.DEFAULT_SESSION_MAP_NAME = JLocalSessionStore.DEFAULT_SESSION_MAP_NAME;
module.exports = LocalSessionStore;